@@ -17,6 +17,7 @@ from logging import getLogger
 
 from .utils import PY2
 from .utils import PY3
+from .utils import PY37plus
 from .utils import Sentinel
 from .utils import basestring
 from .utils import force_bind
@@ -39,9 +40,24 @@ try:
 except ImportError:
     ClassType = type
 
+try:
+    from inspect import isasyncgenfunction
+except ImportError:
+    isasyncgenfunction = None
+
+try:
+    from inspect import iscoroutinefunction
+
+    def isasyncfunction(obj):
+        if isasyncgenfunction is None:
+            return iscoroutinefunction(obj)
+        else:
+            return isasyncgenfunction(obj) or iscoroutinefunction(obj)
+except ImportError:
+    isasyncfunction = None
 
 __all__ = 'weave', 'Aspect', 'Proceed', 'Return', 'ALL_METHODS', 'NORMAL_METHODS', 'ABSOLUTELY_ALL_METHODS'
-__version__ = '1.4.2'
+__version__ = '1.5.0'
 
 logger = getLogger(__name__)
 logdebug = logf(logger.debug)
@@ -159,8 +175,14 @@ class Aspect(object):
         self.bind = bind
 
     def __call__(self, cutpoint_function):
-        if isgeneratorfunction(cutpoint_function):
-            if PY3:
+        if isasyncfunction is not None and isasyncfunction(cutpoint_function):
+            from aspectlib.py35support import decorate_advising_asyncgenerator_py35
+            return decorate_advising_asyncgenerator_py35(self.advising_function, cutpoint_function, self.bind)
+        elif isgeneratorfunction(cutpoint_function):
+            if PY37plus:
+                from aspectlib.py35support import decorate_advising_generator_py35
+                return decorate_advising_generator_py35(self.advising_function, cutpoint_function, self.bind)
+            elif PY3:
                 from aspectlib.py3support import decorate_advising_generator_py3
                 return decorate_advising_generator_py3(self.advising_function, cutpoint_function, self.bind)
             else:
@@ -227,7 +249,7 @@ class Aspect(object):
                                                     else:
                                                         result = None
                                                     break
-                                except BaseException as exc:
+                                except BaseException as exc:  # noqa
                                     advice = advisor.throw(*sys.exc_info())
                                 else:
                                     try:
@@ -244,6 +266,7 @@ class Aspect(object):
                                 raise UnacceptableAdvice("Unknown advice %s" % advice)
                     finally:
                         advisor.close()
+
                 return mimic(advising_generator_wrapper, cutpoint_function)
         else:
             def advising_function_wrapper(*args, **kwargs):
@@ -278,6 +301,7 @@ class Aspect(object):
                             raise UnacceptableAdvice("Unknown advice %s" % advice)
                 finally:
                     advisor.close()
+
             return mimic(advising_function_wrapper, cutpoint_function)
 
 
@@ -325,6 +349,7 @@ class ObjectBag(object):
             self._objects[id(obj)] = obj
             return False
 
+
 BrokenBag = type('BrokenBag', (), dict(has=lambda self, obj: False))()
 
 
@@ -337,6 +362,7 @@ class EmptyRollback(object):
 
     rollback = __call__ = __exit__
 
+
 Nothing = EmptyRollback()
 
 
@@ -539,11 +565,12 @@ def weave_instance(instance, aspect, methods=NORMAL_METHODS, lazy=False, bag=Bro
 
     def fixup(func):
         return func.__get__(instance, type(instance))
+
     fixed_aspect = aspect + [fixup] if isinstance(aspect, (list, tuple)) else [aspect, fixup]
 
     for attr in dir(instance):
-        func = getattr(instance, attr)
         if method_matches(attr):
+            func = getattr(instance, attr)
             if ismethod(func):
                 if hasattr(func, '__func__'):
                     realfunc = func.__func__
@@ -572,8 +599,8 @@ def weave_module(module, aspect, methods=NORMAL_METHODS, lazy=False, bag=BrokenB
              module, aspect, methods, lazy, options)
 
     for attr in dir(module):
-        func = getattr(module, attr)
         if method_matches(attr):
+            func = getattr(module, attr)
             if isroutine(func):
                 entanglement.merge(patch_module_function(module, func, aspect, force_name=attr, **options))
             elif isclass(func):
@@ -615,9 +642,10 @@ def weave_class(klass, aspect, methods=NORMAL_METHODS, subclasses=True, lazy=Fal
         def __init__(self, *args, **kwargs):
             super(SubClass, self).__init__(*args, **kwargs)
             for attr in dir(self):
-                func = getattr(self, attr, None)
-                if method_matches(attr) and attr not in wrappers and isroutine(func):
-                    setattr(self, attr, _checked_apply(aspect, force_bind(func)).__get__(self, SubClass))
+                if method_matches(attr) and attr not in wrappers:
+                    func = getattr(self, attr, None)
+                    if isroutine(func):
+                        setattr(self, attr, _checked_apply(aspect, force_bind(func)).__get__(self, SubClass))
 
         wrappers = {
             '__init__': _checked_apply(aspect, __init__) if method_matches('__init__') else __init__
